package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.ApiUser;
import com.qmth.distributed.print.business.bean.dto.*;
import com.qmth.distributed.print.business.bean.result.WorkResult;
import com.qmth.distributed.print.business.entity.*;
import com.qmth.distributed.print.business.enums.ExamStatusEnum;
import com.qmth.distributed.print.business.enums.ReviewStatusEnum;
import com.qmth.distributed.print.business.mapper.ExamTaskMapper;
import com.qmth.distributed.print.business.service.*;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 命题任务 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class ExamTaskServiceImpl extends ServiceImpl<ExamTaskMapper, ExamTask> implements ExamTaskService {

    @Autowired
    private ExamTaskTempService examTaskTempService;

    @Autowired
    private BasicExamRuleService basicExamRuleService;

    @Autowired
    private ExamTaskReviewLogService examTaskReviewLogService;

    @Autowired
    private ExamTaskDetailService examTaskDetailService;

    @Resource
    ExamTaskMapper examTaskMapper;

    @Override
    public List<ExamTask> listByCourseCode(Long schoolId, String code) {
        QueryWrapper<ExamTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamTask::getSchoolId, schoolId).eq(ExamTask::getCourseCode, code);
        return this.list(queryWrapper);
    }

    @Override
    public IPage<ExamTaskDto> list(Long schoolId, Boolean enable, String status, Long cardRuleId, String courseCode, String paperNumber, Long startTime, Long endTime, Integer pageNumber, Integer pageSize) {
        Page<ExamTaskDto> page = new Page<>(pageNumber, pageSize);
        IPage<ExamTaskDto> examTaskDtoIPage = this.baseMapper.listPage(page, schoolId, enable, status, cardRuleId, courseCode, paperNumber, startTime, endTime);
        return examTaskDtoIPage;
    }

    @Override
    public List<String> listPaperNumber(Long schoolId, String param) {
        QueryWrapper<ExamTask> queryWrapper = new QueryWrapper<>();
        List<String> list = new ArrayList<>();
        if (StringUtils.isNotBlank(param)) {
            queryWrapper.lambda().eq(ExamTask::getSchoolId, schoolId)
                    .and(i -> i.like(ExamTask::getCourseCode, param).or().like(ExamTask::getCourseName, param));
        }
        List<ExamTask> examTasks = this.list(queryWrapper);
        if (examTasks != null && examTasks.size() > 0) {
            list = examTasks.stream().map(m -> m.getPaperNumber()).distinct().collect(Collectors.toList());
        }
        return list;
    }

    @Override
    public List<BlurryUserDto> listUser(Long schoolId, String param) {
        return this.baseMapper.listUser(schoolId, param);
    }

    @Override
    public boolean assignUser(ExamTask examTask) {
        if (examTask.getId() == null) {
            throw ExceptionResultEnum.ERROR.exception("参数有误");
        }

        UpdateWrapper<ExamTask> updateWrapper = new UpdateWrapper<>();
        ExamTask task = this.getById(examTask.getId());
        // 指派命题老师
        if (task.getStatus().name().equals(ExamStatusEnum.NEW.name())) {
            if (!task.getEnable()) {
                throw ExceptionResultEnum.ERROR.exception("命题任务已禁用，不能指派命题老师");
            }
            updateWrapper.lambda().set(ExamTask::getUserId, examTask.getUserId()).set(ExamTask::getStatus, ExamStatusEnum.READY).eq(ExamTask::getId, examTask.getId());
        }

        // 更改命题老师
        if (task.getStatus().name().equals(ExamStatusEnum.READY.name())) {
            if (task.getEnable()) {
                throw ExceptionResultEnum.ERROR.exception("命题任务禁用后，才能更改命题老师");
            }
            updateWrapper.lambda().set(ExamTask::getUserId, examTask.getUserId()).eq(ExamTask::getId, examTask.getId());
        }

        return this.update(updateWrapper);
    }

    @Transactional
    @Override
    public boolean enable(ExamTask examTask) {
        if (examTask.getId() == null) {
            throw ExceptionResultEnum.ERROR.exception("参数有误");
        }

        ExamTask task = this.getById(examTask.getId());
        if (task.getStatus().name().equals(ExamStatusEnum.FINISH)) {
            throw ExceptionResultEnum.ERROR.exception("命题任务状态为已完成，不能启用/禁用操作");
        }
        UpdateWrapper<ExamTask> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamTask::getEnable, examTask.getEnable()).eq(ExamTask::getId, examTask.getId());

        // 解除题卡绑定
        if (task.getEnable()) {
            examTaskDetailService.resetCardId(examTask.getId());
        }
        return this.update(updateWrapper);
    }

    @Override
    public boolean saveExamTask(ExamTask examTask) {
        // todo paperNumber生成问题， 必填字段校验
        QueryWrapper<BasicExamRule> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasicExamRule::getSchoolId, examTask.getSchoolId());
        BasicExamRule basicExamRule = basicExamRuleService.getOne(queryWrapper);
        if (basicExamRule == null) {
            throw ExceptionResultEnum.ERROR.exception("通用规则未设置");
        }
        examTask.setStatus(ExamStatusEnum.NEW);
        examTask.setReview(basicExamRule.getReview());
        return this.save(examTask);
    }

    @Override
    public List<ExamTaskImportDto> importFile(MultipartFile file, String md5, ApiUser apiUser) {
        // 导入文件
        return null;
    }

    @Override
    public boolean saveBatch(ExamTask task) {
        if (StringUtils.isBlank(task.getBatchNo())) {
            throw ExceptionResultEnum.ERROR.exception("batchNo不能为空");
        }
        QueryWrapper<ExamTaskTemp> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamTaskTemp::getBatchNo, task.getBatchNo());
        List<ExamTaskTemp> examTaskTemps = examTaskTempService.list(queryWrapper);
        if (examTaskTemps.isEmpty()) {
            throw ExceptionResultEnum.ERROR.exception("没有可保存数据");
        }
        List<ExamTask> list = new ArrayList<>();
        for (ExamTaskTemp examTaskTemp : examTaskTemps) {
            ExamTask examTask = new ExamTask();
            examTask.setSchoolId(task.getSchoolId());
            examTask.setCourseCode(examTaskTemp.getCourseCode());
            examTask.setCourseName(examTaskTemp.getCourseName());
            examTask.setSpecialty(examTaskTemp.getSpecialty());
            examTask.setPaperNumber(examTaskTemp.getPaperNumber());
            examTask.setStartTime(task.getStartTime());
            examTask.setEndTime(task.getEndTime());
            examTask.setCardRuleId(task.getCardRuleId());
            examTask.setStatus(ExamStatusEnum.NEW);
            examTask.setBatchNo(task.getBatchNo());
            examTask.setCreateId(task.getCreateId());
            examTask.setCreateTime(System.currentTimeMillis());
            String userId = task.getUsers().stream().map(m -> {
                if (examTaskTemp.getCourseCode().equals(m.get("courseCode"))) {
                    return m.get("userId");
                }
                return "";
            }).findFirst().get();
            if (StringUtils.isNotBlank(userId)) {
                examTask.setUserId(Long.valueOf(userId));
            }
            list.add(examTask);
        }
        return this.saveBatch(list);
    }

    @Override
    public IPage<ExamTaskDto> listTaskApply(Long schoolId, String auditStatus, String reviewStatus, Long cardRuleId, String courseCode, String paperNumber, Long startTime, Long endTime, Integer pageNumber, Integer pageSize) {
        Page<ExamTaskDto> page = new Page<>(pageNumber, pageSize);
        String[] strings = new String[]{ExamStatusEnum.NEW.name(), ExamStatusEnum.FINISH.name()};
        IPage<ExamTaskDto> examTaskDtoIPage = this.baseMapper.listTaskApply(page, schoolId, auditStatus, reviewStatus, cardRuleId, courseCode, paperNumber, startTime, endTime, strings);
        return examTaskDtoIPage;
    }

    @Override
    public IPage<ExamTaskDto> listTaskReviewUnaudited(Long schoolId, String courseCode, String paperNumber, Long userId, Long cardRuleId, Long startTime, Long endTime, Integer pageNumber, Integer pageSize) {
        Page<ExamTaskDto> page = new Page<>(pageNumber, pageSize);
        IPage<ExamTaskDto> examTaskDtoIPage = this.baseMapper.listTaskReviewUnaudited(page, schoolId, courseCode, paperNumber, userId, cardRuleId, startTime, endTime);
        return examTaskDtoIPage;
    }

    @Override
    public IPage<ExamTaskDto> listTaskReviewAudited(Long schoolId, String reviewStatus, String courseCode, String paperNumber, Long userId, Long cardRuleId, Long startTime, Long endTime, Integer pageNumber, Integer pageSize) {
        Page<ExamTaskDto> page = new Page<>(pageNumber, pageSize);
        IPage<ExamTaskDto> examTaskDtoIPage = this.baseMapper.listTaskReviewAudited(page, schoolId, reviewStatus, courseCode, paperNumber, userId, cardRuleId, startTime, endTime);
        return examTaskDtoIPage;
    }

    @Transactional
    @Override
    public boolean taskReviewSave(ExamTaskReviewLog taskReviewLog) {
        // 审核日志
        examTaskReviewLogService.save(taskReviewLog);

        // 更新命题任务状态
        UpdateWrapper<ExamTask> updateWrapper = new UpdateWrapper<>();
        ExamStatusEnum statusEnum = null;
        if (taskReviewLog.getReviewStatus().name().equals(ReviewStatusEnum.PASS.name())) {
            statusEnum = ExamStatusEnum.FINISH;
        } else {
            statusEnum = ExamStatusEnum.STAGE;
        }
        updateWrapper.lambda().set(ExamTask::getStatus, statusEnum).set(ExamTask::getReviewStatus, taskReviewLog.getReviewStatus()).eq(ExamTask::getId, taskReviewLog.getExamTaskId());
        return this.update(updateWrapper);
    }

    @Transactional
    @Override
    public boolean taskReviewSaveBatch(ExamTaskReviewLog taskReviewLog) {
        ExamStatusEnum statusEnum = null;
        if (taskReviewLog.getReviewStatus().name().equals(ReviewStatusEnum.PASS.name())) {
            statusEnum = ExamStatusEnum.FINISH;
        } else {
            statusEnum = ExamStatusEnum.STAGE;
        }
        for (Long examTaskId : taskReviewLog.getExamTaskIds()) {
            // 审核日志
            taskReviewLog.setExamTaskId(examTaskId);
            examTaskReviewLogService.save(taskReviewLog);

            // 更新命题任务状态
            UpdateWrapper<ExamTask> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().set(ExamTask::getStatus, statusEnum).set(ExamTask::getReviewStatus, taskReviewLog.getReviewStatus()).eq(ExamTask::getId, taskReviewLog.getExamTaskId());
            this.update(updateWrapper);
        }
        return true;
    }

    @Override
    public IPage<ExamTaskDetailDto> listTaskPaper(Long schoolId, String courseCode, String paperNumber, Long startTime, Long endTime, Integer pageNumber, Integer pageSize) {
        Page<ExamTaskDetailDto> page = new Page<>(pageNumber, pageSize);
        IPage<ExamTaskDetailDto> examTaskDtoIPage = this.baseMapper.listTaskPaper(page, schoolId, courseCode, paperNumber, startTime, endTime);
        return examTaskDtoIPage;
    }

    @Override
    public List<RelatePaperDto> listPaperTypes(Long schoolId, String courseCode) {
        List<RelatePaperDto> list = this.baseMapper.listPaperNumbers(schoolId, courseCode);
        for (RelatePaperDto relatePaperDto : list) {
            QueryWrapper<ExamTaskDetail> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(ExamTaskDetail::getExamTaskId, relatePaperDto.getId()).eq(ExamTaskDetail::getEnable, true);
            ExamTaskDetail examTaskDetail = this.examTaskDetailService.getOne(queryWrapper);
            if (examTaskDetail != null) {
                String unexposedPaperType = examTaskDetail.getUnexposedPaperType();
                relatePaperDto.setPaperTypes(StringUtils.isBlank(unexposedPaperType) ? null : Arrays.asList(unexposedPaperType.split("/")));
            }
        }
        return list;
    }

    @Transactional
    @Override
    public boolean saveExamTaskDetail(ExamTaskDetail examTaskDetail) {
        if (examTaskDetail.getExamTaskId() == null) {
            throw ExceptionResultEnum.ERROR.exception("命题任务ID不能为空");
        }
        if (StringUtils.isBlank(examTaskDetail.getOperateType())) {
            throw ExceptionResultEnum.ERROR.exception("保存类型不能为空");
        }
        // 提交时，校验
        if (ExamStatusEnum.SUBMIT.name().equals(examTaskDetail.getOperateType())) {
            if (StringUtils.isBlank(examTaskDetail.getPaperType())) {
                throw ExceptionResultEnum.ERROR.exception("试卷类型不能为空");
            }
            if (StringUtils.isBlank(examTaskDetail.getPaperAttachmentIds())) {
                throw ExceptionResultEnum.ERROR.exception("试卷ID不能为空");
            }
            if (examTaskDetail.getCardId() == null) {
                throw ExceptionResultEnum.ERROR.exception("题卡ID不能为空");
            }
            if (StringUtils.isBlank(examTaskDetail.getPaperConfirmAttachmentIds())) {
                throw ExceptionResultEnum.ERROR.exception("入库审核ID不能为空");
            }
        }

        // 更新examTask状态status
        ExamTask examTask = this.getById(examTaskDetail.getExamTaskId());
        if (examTask.getStatus().name().equals(ExamStatusEnum.SUBMIT.name()) && !examTask.getReview()) {
            examTask.setStatus(ExamStatusEnum.FINISH);
        }

        UpdateWrapper<ExamTask> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamTask::getStatus, ExamStatusEnum.valueOf(examTaskDetail.getOperateType())).set(ExamTask::getReviewStatus, null).eq(ExamTask::getId, examTask.getId());
        this.update(updateWrapper);

        QueryWrapper<ExamTaskDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamTaskDetail::getExamTaskId, examTaskDetail.getExamTaskId());
        ExamTaskDetail detail = examTaskDetailService.getOne(queryWrapper);
        if (detail != null) {
            examTaskDetail.setId(detail.getId());
        }

        return examTaskDetailService.saveOrUpdate(examTaskDetail);
    }

    @Override
    public boolean status(ExamTask examTask) {
        if (examTask.getId() == null) {
            throw ExceptionResultEnum.ERROR.exception("参数有误");
        }
        ExamTask task = this.getById(examTask.getId());
        UpdateWrapper<ExamTask> updateWrapper = new UpdateWrapper<>();
        // 撤回
        if (examTask.getStatus().name().equals(ExamStatusEnum.CANCEL.name())) {
            /*if (!task.getStatus().name().equals(ExamStatusEnum.FINISH.name())) {
                throw ExceptionResultEnum.ERROR.exception("命题任务状态为已完成，才能撤回");
            }*/
            updateWrapper.lambda().set(ExamTask::getStatus, examTask.getStatus()).eq(ExamTask::getId, examTask.getId());
            return this.update(updateWrapper);
        }
        return false;
    }

    @Override
    public ExamTaskDetailCardDto applyGetOne(Long examTaskId) {
        ExamTaskDetailCardDto examTaskDetailDto = this.baseMapper.applyGetOne(examTaskId);
        return examTaskDetailDto;
    }

    /**
     * 查询我的工作台
     *
     * @param iPage
     * @param userId
     * @param schoolId
     * @param status
     * @return
     */
    @Override
    public IPage<WorkResult> queryByMyWork(IPage<Map> iPage, Long userId, Long schoolId, ExamStatusEnum status) {
        return examTaskMapper.queryByMyWork(iPage, userId, schoolId, Objects.nonNull(status) ? status.name() : null);
    }

    /**
     * 查询我的工作台-审核待办
     *
     * @param iPage
     * @param userId
     * @param schoolId
     * @param status
     * @return
     */
    @Override
    public IPage<WorkResult> queryByMyWorkSubmit(IPage<Map> iPage, Long userId, Long schoolId, ExamStatusEnum status) {
        return examTaskMapper.queryByMyWorkSubmit(iPage, userId, schoolId, Objects.nonNull(status) ? status.name() : null);
    }
}
